/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/* TMP116's 16 bit result contains 7 fractional bits, right justified.
 * This routine can convert all of them to string without losing
 * precision, but it's wasteful to use the CPU for this when we
 * won't display all of the characters.
 *
 * This routine may not be more efficient than printf depending
 * on architecture. printf features may be disabled or restricted
 * in your project by default. In CCS 7.3, open the Properties
 * window for your project and navigate to 'CCS Build > MSP430
 * Compiler > Advanced Options > Language Options' to configure
 * printf features.
 */

//#include <stdio.h>
//#include <stdlib.h>
//#define DEBUG

#define frac_bits 4 //number of fractional bits to convert to string
#define dscrd_bits 3 //put total_bits - frac_bits here
#define lsb_c 625 //put 5^frac_bits here
#define lsb_f 1125 //put lsb_c*9/5 here
#define ofs_f 32e4 //put 32e(frac_bits) here
#define st_char_len 10 // sign + 3 + decimal + frac_bits + unit
/* conveniently, the number of fractional bits is also the distance
 * (in base 10 digits) from 1
 */

/* if you need to use frac_bits > 6 and fahrenheit, then data type
 * 'long' isn't big enough. change var 'it' to 'long long'
 */

char st[st_char_len];

char* tempstring(signed char HiByte, signed char LoByte, char fahrenheit){
    char i;
    long it;

    //combine 2 bytes
    it = (long) HiByte << 8 | (LoByte & 0xFF);
    it >>= dscrd_bits;
    //LSB
    if (fahrenheit){
        it *= lsb_f;
        it += ofs_f;
        st[st_char_len-1] = 'F';
    }
    else {
        it *= lsb_c;
        st[st_char_len-1] = 'C';
    }
    #ifdef DEBUG
    printf("it: %lli i: %i\n",it,i);
    #endif
    //sign
    if (it<0) {
        st[0] = 0x2D;
        it *= -1;
    }
    else st[0] = 0x20;
    //decimal
    st[4] = 0x2E;
    for(i=st_char_len-2;i > 0;i--){
        #ifdef DEBUG
        printf("it: %lli i: %i\n",it,i);
        #endif
        if (i!=4){
            st[i] = it % 10 + 0x30;
            it /= 10;
        }
    }
    //fix leading zeroes
    for (i=1; i < 3; i++){
        //if equals zero
        if (st[i]==0x30) {
            //if previous equals minus
            if (st[i-1]==0x2D) {
                st[i] = 0x2D;
                st[i-1] = 0x20;
            }
            //else if previous equals space
            else if (st[i-1]==0x20)
            st[i] = 0x20;
        }
        else break;
    }
    return st;
}
